<?php
/*
  $Id: attributeManager.php,v 3.6.0

  by @raiwa
  raiwa@phoenixcartaddons.com
  www.phoenixcartaddons.com

  Copyright © 2021 Rainer Schmied

  based on attributeManager.php,v 1.0 21/02/06 Sam West$

  CE Phoenix, E-Commerce made Easy
  https://phoenixcart.org

  Released under the GNU General Public License

  Copyright © 2006 Kangaroo Partners
  http://kangaroopartners.com
  osc@kangaroopartners.com
*/

class attributeManagerInstant extends attributeManager {

	/**
	 * @access private
	 */
	var $intPID;

	/**
	 * __construct() assigns pid, calls the parent construct, registers page actions
	 * @access public
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $intPID int
	 * @return void
	 */
	function __construct($intPID) {

		parent::attributeManager();

		$this->intPID = (int)$intPID;

		$this->registerPageAction('addAttributeToProduct','addAttributeToProduct');
		$this->registerPageAction('addOptionValueToProduct','addOptionValueToProduct');
		$this->registerPageAction('addNewOptionValueToProduct','addNewOptionValueToProduct');
		$this->registerPageAction('removeOptionFromProduct','removeOptionFromProduct');
		$this->registerPageAction('removeOptionValueFromProduct','removeOptionValueFromProduct');
		$this->registerPageAction('update','update');

    $this->registerPageAction('moveOption','moveOption');
    $this->registerPageAction('moveOptionValue','moveOptionValue');

//----------------------------
// Change: Add image attributes function for AM
// @author @raiwa
// Function: register PageActions for Image options
//-----------------------------

		$this->registerPageAction('addImageAttributeToProduct','addImageAttributeToProduct');
		$this->registerPageAction('removeImageAttributeToProduct','removeImageAttributeToProduct');
//----------------------------
// EOF Change: image attributes for AM
//-----------------------------

//----------------------------
// Change: Add download attributes function for AM
// @author Urs Nyffenegger ak mytool
// Function: register PageActions for Download options
//-----------------------------

		$this->registerPageAction('addDownloadAttributeToProduct','addDownloadAttributeToProduct');
		$this->registerPageAction('updateDownloadAttributeToProduct','updateDownloadAttributeToProduct');
		$this->registerPageAction('removeDownloadAttributeToProduct','removeDownloadAttributeToProduct');
//----------------------------
// EOF Change: download attributes for AM
//-----------------------------


	}

	//----------------------------------------------- page actions

	/**
	 * Adds the selected attribute to the current product
	 * @access public
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $get $_GET
	 * @return void
	 */
	function addAttributeToProduct($get) {

		$this->getAndPrepare('option_id', $get, $optionId);
		$this->getAndPrepare('option_value_id', $get, $optionValueId);
		if (isset($get['price'])) $this->getAndPrepare('price', $get, $price);
		if (isset($get['prefix'])) $this->getAndPrepare('prefix', $get, $prefix);

		if ( (empty($price)) || ($price == '0') ) {
			$price = '0.0000';
		} else {
			if ( (empty($prefix)) || ($prefix == ' ') ) {
				$prefix = '+';
			}
		}
		if (empty($prefix)) {
			$prefix = ' ';
		}

		$data = ['products_id' => $this->intPID,
              'options_id' => $optionId,
       'options_values_id' => $optionValueId,
		'options_values_price' => $price,
            'price_prefix' => $prefix];

    if (AM_USE_MPW == 'true') {
      if (isset($get['weight'])) $this->getAndPrepare('weight', $get, $weight);
      if (isset($get['weight_prefix'])) $this->getAndPrepare('weight_prefix', $get, $weight_prefix);

      if ( (empty($weight)) || ($weight == '0') ) {
        $weight = '0.0000';
      } else {
        if ( (empty($weight_prefix)) || ($weight_prefix == ' ') ) {
          $weight_prefix = '+';
        }
      }
      if (empty($weight_prefix)) {
        $weight_prefix = ' ';
      }

      $data['option_weight'] = $weight;
      $data['option_weight_prefix'] = $weight_prefix;
    }

// options images added by @raiwa
    if (AM_USE_IMAGE == 'true') {
      if (isset($get['image'])) $this->getAndPrepare('image', $get, $image);
      if ( (empty($image)) || ($image == '0') ) {
        $image = '';
      }
      $data['option_image'] = $image;
    }

		// wholesale added by @BrockleyJohn
		if (AM_USE_WHOLESALE == 'true') {
			$this->getAndPrepare('ws_price', $get, $ws_price);
			if ( (empty($ws_price)) || ($ws_price=='0') ) {
				$ws_price='0.0000';
			}
      $data['options_values_ws_price'] = $ws_price;
		}

    if (isset($get['sortOrder'])) $this->getAndPrepare('sortOrder', $get, $sortOrder);
    // changes by mytool
    // get highest sort order value
    $insertIndex = -1;
    $result = $this -> getSortedProductAttributes('sort_order');
    // search for the current Sort Order where the new value needs to be added
    $i = -1;
    foreach($result as $key => $val) {
      $i++;
      if ( $val['options_id'] == $optionId ) {
        $insertIndex = $i;
      }
    }

    // if InsertIndex is still -1 then this is a new option and will be added at the end
    if ($insertIndex > -1) {
      $i = -1;
      $newArray = [];

      for ($n=0; $n < count($result) ; $n++) {
        $i++;
          if ($i == $insertIndex) {
          $i++;
            $data['sort_order'] = $i;
            $newArray[$i] = $result[$n];
          } else {
            $result[$n]['sort_order'] = $i;
            $newArray[$i] = $result[$n];
          }
        }

      $this->updateSortedProductArray($newArray);

    } else {
      $lastrow = end($result);
      $data['sort_order'] = ((isset($lastrow['sort_order']) && $lastrow['sort_order'] != '0')? (int)$lastrow['sort_order'] + 1 : 1);
    }
    // EO mytool

		$GLOBALS['db']->perform("products_attributes", $data);
	}

	/**
	 * Adds an existing option value to a product
	 * @see addAttributeToProduct()
	 */
	function addOptionValueToProduct($get) {
		$this->addAttributeToProduct($get);
	}

	/**
	 * Adds a new option value to the database then assigns it to the product
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $get $_GET
	 * @return void
	 */
	function addNewOptionValueToProduct($get) {
		$returnInfo = $this->addOptionValue($get);
		$get['option_value_id'] = $returnInfo['selectedOptionValue'];
		$this->addAttributeToProduct($get);
	}

	/**
	 * Removes a specific option and its option values from the current product
	 * @access public
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $get $_GET
	 * @return void
	 */
	function removeOptionFromProduct($get) {
		$this->getAndPrepare('option_id', $get, $optionId);
		$GLOBALS['db']->query("DELETE
                  FROM products_attributes
                  WHERE options_id = '" . $optionId . "'
                    AND products_id = '" . $this->intPID . "'");

		$this->updateSortOrder();
	}

	/**
	 * Removes a specific option value from a the current product
	 * @access public
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $get $_GET
	 * @return void
	 */
	function removeOptionValueFromProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$GLOBALS['db']->query("DELETE
                            FROM products_attributes
                            WHERE options_id = '" . $optionId . "'
                              AND options_values_id = '" . $optionValueId . "'
                              AND products_id = '" . $this->intPID . "'");

		$this->updateSortOrder();
	}


//----------------------------
// Change: Add Image attributes function for AM
// @author Urs Nyffenegger ak mytool
// Function: Add, delete and edit image options
//-----------------------------

	function addImageAttributeToProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$this->getAndPrepare('option_image',$get,$option_image);
		$this->getAndPrepare('products_attributes_id',$get,$products_attributes_id);

		$GLOBALS['db']->query("UPDATE products_attributes
                            SET option_image='" . $option_image . "'
                            WHERE products_attributes_id = '" . $products_attributes_id . "'
                              AND options_id = '" . $optionId . "'
                              AND options_values_id = '" . $optionValueId . "'
                              AND products_id = '" . $this->intPID . "'" );
	}

	function removeImageAttributeToProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$this->getAndPrepare('products_attributes_id',$get,$products_attributes_id);

		$GLOBALS['db']->query("UPDATE products_attributes
                            SET option_image = ''
                            WHERE products_attributes_id = '" . $products_attributes_id . "'
                              AND options_id = '" . $optionId . "'
                              AND options_values_id = '" . $optionValueId . "'
                              AND products_id = '" . $this->intPID . "'" );
	}

///----------------------------
// EOF Change: Image attributes for AM
//-----------------------------


//----------------------------
// Change: Add download attributes function for AM
// @author Urs Nyffenegger ak mytool
// Function: Add, delete and edit Download options
//-----------------------------

	function updateDownloadAttributeToProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$this->getAndPrepare('products_attributes_filename',$get,$products_attributes_filename);
		$this->getAndPrepare('products_attributes_maxdays',$get,$products_attributes_maxdays);
		$this->getAndPrepare('products_attributes_maxcount',$get,$products_attributes_maxcount);
		$this->getAndPrepare('products_attributes_id',$get,$products_attributes_id);

		$GLOBALS['db']->query("UPDATE products_attributes_download
                            SET products_attributes_filename = '" . $products_attributes_filename ."', products_attributes_maxdays = '" . $products_attributes_maxdays . "', products_attributes_maxcount = '" . $products_attributes_maxcount . "'
                            WHERE products_attributes_id = '" . $products_attributes_id . "'");
	}

	function addDownloadAttributeToProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$this->getAndPrepare('products_attributes_filename',$get,$products_attributes_filename);
		$this->getAndPrepare('products_attributes_maxdays',$get,$products_attributes_maxdays);
		$this->getAndPrepare('products_attributes_maxcount',$get,$products_attributes_maxcount);
		$this->getAndPrepare('products_attributes_id',$get,$products_attributes_id);

		$GLOBALS['db']->query("INSERT INTO products_attributes_download (products_attributes_id, products_attributes_filename, products_attributes_maxdays, products_attributes_maxcount)
                  VALUES('" . $products_attributes_id . "','" . $products_attributes_filename . "', '" . $products_attributes_maxdays . "', '" . $products_attributes_maxcount . "')");
	}

	function removeDownloadAttributeToProduct($get) {
		$this->getAndPrepare('option_id',$get,$optionId);
		$this->getAndPrepare('option_value_id',$get,$optionValueId);
		$this->getAndPrepare('products_attributes_id',$get,$products_attributes_id);

		$GLOBALS['db']->query("DELETE
                            FROM products_attributes_download
                            WHERE products_attributes_id = '" . $products_attributes_id . "'");
	}
//----------------------------
// EOF Change: download attributes for AM
//-----------------------------

	/**
	 * Updates the price and prefix in the products attribute table
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @param $get $_GET
	 * @return void
	 */
	function update($get) {

		$this->getAndPrepare('option_id', $get, $optionId);
		$this->getAndPrepare('option_value_id', $get, $optionValueId);
		$this->getAndPrepare('price', $get, $price);
		$this->getAndPrepare('prefix', $get, $prefix);
		$this->getAndPrepare('sortOrder', $get, $sortOrder);

		if ( (empty($price)) || ($price=='0') ) {
		  $price='0.0000';
		} else {
		  if ( (empty($prefix)) || ($prefix == ' ') ) {
        $prefix = '+';
		  }
		}

		$data = ['options_values_price' => $price,
                     'price_prefix' => $prefix];

    if (AM_USE_MPW == 'true') {
      $this->getAndPrepare('weight', $get, $weight);
      $this->getAndPrepare('weight_prefix', $get, $weight_prefix);

      if ( (empty($weight)) || ($weight == '0') ) {
        $weight='0.0000';
      } else {
        if ( (empty($weight_prefix)) || ($weight_prefix == ' ') ) {
          $weight_prefix='+';
        }
      }

      $data['option_weight'] = $weight;
      $data['option_weight_prefix'] = $weight_prefix;
    }

// options images added by @raiwa
    if (AM_USE_IMAGE == 'true') {
      $this->getAndPrepare('option_image', $get, $option_image);
      if ( (empty($option_image)) || ($option_image == '0' ) ) {
        $option_image='';
      }
      $data['option_image'] = $option_image;
    }

		// wholesale added by @BrockleyJohn
		if (AM_USE_WHOLESALE == 'true') {
			$this->getAndPrepare('ws_price', $get, $ws_price);
			if ( (empty($ws_price)) || ($ws_price=='0') ) {
				$ws_price='0.0000';
			}
			$data['options_values_ws_price'] = $ws_price;
		}

		$GLOBALS['db']->perform("products_attributes", $data, 'update', "products_id = '" . $this->intPID . "' AND options_id = '" . $optionId . "' AND options_values_id = '" . $optionValueId . "'");

	}

	//----------------------------------------------- page actions end

	/**
	 * Returns all or the options and values in the database
	 * @access public
	 * @author Sam West aka Nimmit - osc@kangaroopartners.com
	 * @return array
	 */
	function getAllProductOptionsAndValues($reset = false) {
		if ( 0 === count($this->arrAllProductOptionsAndValues) || true === $reset ) {
			$this->arrAllProductOptionsAndValues = [];

			$allOptionsAndValues = $this->getAllOptionsAndValues();
//----------------------------
// Change: Add download attributes function for AM
// @author Urs Nyffenegger ak mytool
// Function: change query string to add the Download Table fields
//-----------------------------
			$queryString = "SELECT pa.*, pad.products_attributes_filename, pad.products_attributes_maxdays, pad.products_attributes_maxcount
                        FROM products_attributes AS pa INNER JOIN products_options po
                          ON pa.options_id = po.products_options_id";
			$queryString .= "   LEFT JOIN products_attributes_download pad
                            ON pa.products_attributes_id = pad.products_attributes_id";
			$queryString .= " WHERE products_id = '" . $this->intPID . "'
                          AND language_id = '" . (int)$this->getSelectedLanaguage() . "'
                        ORDER BY sort_order";
//----------------------------
// EOF Change: download attributes for AM
//-----------------------------
			$query = $GLOBALS['db']->query($queryString);

			$optionsId = null;
			while ($res = $query->fetch_assoc() ) {


				if ($res['options_id'] != $optionsId) {
					$optionsId = $res['options_id'];
					$this->arrAllProductOptionsAndValues[$optionsId]['name'] = $allOptionsAndValues[$optionsId]['name'];
				}
				// option types added by @raiwa 
				if ( AM_USE_OPTION_TYPES == 'true' && $res['options_values_id'] == OPTIONS_TYPE_TEXT_ID ) {
          $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['name'] = $allOptionsAndValues[$optionsId]['values'][$res['options_values_id']] ?? 'CUSTOMER-INPUT';
        } else {
          $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['name'] = $allOptionsAndValues[$optionsId]['values'][$res['options_values_id']] ?? '';
        }
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['price'] = $res['options_values_price'];
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['prefix'] = $res['price_prefix'];

        if (AM_USE_MPW == 'true') {
          if (!empty($res['option_weight']) ) {
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['weight'] = $res['option_weight'];
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['weight_prefix'] = $res['option_weight_prefix'];
          } else {
            $image_query = $GLOBALS['db']->query("SELECT products_options_values_weight, products_options_values_weight_prefix
                                                   FROM products_options_values
                                                   WHERE products_options_values_id = '" . $res['options_values_id'] . "'
                                                    AND language_id = " . (int)$this->getSelectedLanaguage());
            $res_product_image = $image_query->fetch_assoc();
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['weight'] = $res_product_image['products_options_values_weight']?? '0.00';
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['weight_prefix'] = $res_product_image['products_options_values_weight_prefix']?? '';
          }
        }

        if (AM_USE_IMAGE == 'true') {
          if (!empty($res['option_image']) ) {
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['image'] = $res['option_image'];
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['product_option_image'] = true;
          } else {
            $image_query = $GLOBALS['db']->query("SELECT products_options_values_image
                                                   FROM products_options_values
                                                   WHERE products_options_values_id = '" . $res['options_values_id'] . "'
                                                    AND language_id = " . (int)$this->getSelectedLanaguage());
            $res_product_image = $image_query->fetch_assoc();
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['image'] = $res_product_image['products_options_values_image'];
            $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['product_option_image'] = false;
          }
        }

				// wholesale added by @BrockleyJohn
				if (AM_USE_WHOLESALE == 'true') {
					$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['ws_price'] = $res['options_values_ws_price'];
				}
        
//----------------------------
// Change: Add image attributes function for AM
// @author @raiwa
// Function: get the new Attributes
//-----------------------------
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['products_attributes_id'] = $res['products_attributes_id'];
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['option_image'] = (isset($res['option_image'])?$res['option_image']:'');
//----------------------------
// EOF Change: image attributes for AM
//-----------------------------

//----------------------------
// Change: Add download attributes function for AM
// @author Urs Nyffenegger ak mytool
// Function: get the new Attributes
//-----------------------------
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['products_attributes_id'] = $res['products_attributes_id'];
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['products_attributes_filename'] = $res['products_attributes_filename'];
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['products_attributes_maxdays'] = $res['products_attributes_maxdays'];
				$this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['products_attributes_maxcount'] = $res['products_attributes_maxcount'];
//----------------------------
// EOF Change: download attributes for AM
//-----------------------------

        $this->arrAllProductOptionsAndValues[$optionsId]['values'][$res['options_values_id']]['sortOrder'] = $res['sort_order'];
			}
		}
		return $this->arrAllProductOptionsAndValues;
	}

	function moveOptionUp() {
		$this->moveOption();
	}

	function moveOptionDown() {
		$this->moveOption('down');
	}

	function moveOption($get) {

		$extraValues = $this->getExtraValues($get['gets']);
		$direction = $get['dir'];
		$changes = false;
		$newArray = [];

		// Get current State -- is this necessary? or could we take the getAllProductOptionsAndValues?? i'll see later
		$sortedArray = $this->getSortedProductAttributes( 'sort_order' );

		// now create new array with the optionsID unique
		$i = -1;
		$firstRow = current($sortedArray);
		$start_ID = $firstRow['options_id'];

		foreach ($sortedArray as $key => $val) {
			if ( $val['options_id'] != $start_ID ) {
				$i =  -1;
				$start_ID = $val['options_id'];
			}

			$i++;
			$optionsArray[ $val['options_id'] ][$i] = $val;

		}

		// get position so we can swap
		$positionArray = array_keys($optionsArray);
		$position = array_search( (int)$extraValues['option_id'], $positionArray);

		if ($direction == 'up') {

			if ( $position > 0 ) {
				$changes = true;
				$prevItem = $positionArray[ $position - 1];
				$ThisItem = $positionArray[$position];
				$positionArray[$position] = $prevItem;
				$positionArray[$position - 1] = $ThisItem;
			}
		} else {
			if ( $position < ( count($positionArray)-1 ) ) {
				$changes = true;
				$nextItem = $positionArray[ $position + 1];
				$ThisItem = $positionArray[$position];
				$positionArray[$position] = $nextItem;
				$positionArray[$position + 1] = $ThisItem;
			}
		}

		// set new Sortvalues
		$i =  -1;
		foreach ($positionArray as $key => $val) {
			foreach ($optionsArray[ $val ] as $okey => $oval) {
					$i++;
					$oval['sort_order'] = $i;
					$newArray[$i] = $oval;
			 }
		}

		// update Database
		if($changes){
			$this->updateSortedProductArray($newArray);
		}
	}

	function moveOptionValue($get) {

		$extraValues = $this->getExtraValues($get['gets']);
		$direction = $get['dir'];
		$changes = false;
		$sortedArray = [];
		$newArray = [];

		$sortedArray = $this->getSortedProductAttributes('sort_order');

		$i = -1;

		// filter array
		foreach ($sortedArray as $key => $val) {
   			if ( $val['options_id'] == $extraValues['option_id'] ) {
   				$i++;
   				$newArray[$val['sort_order']] = $val;
   			}
   		}

		// get first and Last Row, so we can determine lowest and higest Sort order value later

		$first = current($newArray);
		$firstSortValue = (int)$first['sort_order'];
		$lastSortValue = $firstSortValue + count($newArray) - 1;

		foreach ($newArray as $key => $val) {
   		if ( $val['products_attributes_id'] == $extraValues['products_attributes_id'] ) {
    		$startSort = $val['sort_order'];
			}
		}

		if ($direction == 'up') {
			// ceiling_ only change if its not the top item
			if ($startSort > (int)$firstSortValue ) {
				$changes = true;
				$newArray[$startSort]['sort_order'] = (int)$startSort - 1;
				$newArray[$startSort-1]['sort_order'] = (int)$startSort;
			}
		} else {
			// ceiling only change if its not the bottom item
			if ( $startSort < (int)$lastSortValue ) {
				$changes = true;
				$newArray[$startSort]['sort_order'] = (int)$startSort + 1;
				$newArray[$startSort+1]['sort_order'] = (int)$startSort;
			}
		}

		// update Database
		if ($changes) {
			$this->updateSortedProductArray($newArray);
		}

	}

	function getExtraValues($gets){
		$arrExtraValues = [];
		$valuePairs = [];

		if (strpos($gets,'|'))  {
			$valuePairs = explode('|',$gets);
    } else {
			$valuePairs[] = $gets;
    }

		foreach ($valuePairs as $pair) {
			if (strpos($pair,':')) {
				list($extraKey, $extraValue) = explode(':',$pair);
				$arrExtraValues[$extraKey] = $extraValue;
			}
    }
		return $arrExtraValues;
	}

	function getSortedProductAttributes( $sortfield ){

		$sortedArray = [];

		$queryString = "SELECT products_attributes_id, options_id, sort_order
                    FROM products_attributes
                    WHERE products_id = " . $this->intPID .	"
                    ORDER BY " . $sortfield . " ASC, options_id ASC";

		$result = amDB::getAll($queryString);
		$i = 0;

		foreach ($result as $key => $val) {
			// set the sorting new
			$val['sort_order'] = $i;
			$sortedArray[$i] = $val;
			$i++;
		}

		return $sortedArray;
	}


	function updateSortedProductArray($newArray){

		foreach ($newArray as $key => $val) {
		  if ( !empty($val['products_attributes_id']) ) {
				$GLOBALS['db']->perform("products_attributes", $val, 'update', 'products_attributes_id = ' . $val['products_attributes_id']);
			}
		}
	}

	function updateSortOrder(){

    $newArray = $this->getSortedProductAttributes('sort_order');
    $this->updateSortedProductArray( $newArray );
	}

}
